/* 
 * The simple src/sink example, with a  manager which 
 * creates new sources on the fly
 */

import eduni.simjava.*;

class Source extends Sim_entity {
  private Sim_port out;
  private int index;
  private int state;

  private boolean isdynamic = false;

  public static final int SRC_OK      = 0;
  public static final int SRC_BLOCKED = 1;

  public Source(String name, int index, int state, boolean isdynamic) {
    super(name);
    this.index = index;
    this.state = state;
    this.isdynamic = isdynamic;
    out = new Sim_port("out");
    add_port(out);
  }

  public void body() {
    Sim_event ev = new Sim_event();
    int i;

    System.out.println("About to do body S");

    if (isdynamic) {
      // Wait for 'startup' event, after which out port will
      // be connected
      sim_wait(ev);
      System.out.println("Started Up");
    }

    for (i=0; i<100; i++) {
      sim_schedule(out,0.0,0);
      sim_wait(ev);
      sim_hold(10.0);
      sim_trace(1,"C Src loop index is "+i);
    }
    System.out.println("Exiting body S");
  }
}

class Sink extends Sim_entity {
  private Sim_port in;
  private int index;
  private int state;

  public static final int SINK_BLOCKED = 0;
  public static final int SINK_OK      = 1;

  public Sink(String name, int index, int state) {
    super(name);
    this.index = index;
    this.state = state;
    in = new Sim_port("in");
    add_port(in);
  }

  public synchronized void new_port(String np) {
    add_port(new Sim_port(np));
  }

  public void body() {
    Sim_event ev = new Sim_event();
    int i = 0;

    System.out.println("About to do body R");
    while(true) {
      i++; if(i>50) break;
      sim_wait(ev);
      int sid = ev.scheduled_by();
      Sim_entity src = Sim_system.get_entity(sid);
      sim_trace(1,"A message arrived from entity :"+src.get_name());

      sim_hold(1.234);
      sim_schedule(in,0.0,1);
    }
    System.out.println("Exiting body S");
  }
}


// Create 10 new sources on the fly
class Manager extends Sim_entity {
  Manager(String name) {
    super(name);
  }
 
  public void body() {
    for (int i=0; i<10; i++) {
      sim_hold(1.0);

      System.out.println("Mgr about to make source");

      /* Add a new entity to the system
       * NOTE this will not be done immediately!!!
       * A 'CREATE' event will be placed on the future
       * queue (at the current sim time)
       */
      Source s = new Source("NewSource"+i, i+2, Source.SRC_OK, true);
      System.out.println("Mgr made source");
      // Sim_system.add(s);

      sim_hold(1.0); // Allow entity to be created
      // System.out.println("Mgr doing something else");
     
      Sink sk = (Sink)Sim_system.get_entity("Receiver");
      sk.new_port("in"+i);
      Sim_system.link_ports("NewSource"+i, "out", "Receiver", "in"+i);

      // Set new source running
      sim_schedule(s.get_id(),0,0);
    }
  }
}

class Example5 {
  public static void main(String args[]) {
    Sim_system.initialise();
    Sim_system.add(new Manager("Manager"));
    Sim_system.add(new Source("Sender", 1, Source.SRC_OK, false));
    Sim_system.add(new Sink("Receiver", 2, Sink.SINK_OK));
    Sim_system.link_ports("Sender", "out", "Receiver", "in");
    Sim_system.run();
    System.out.println("Exiting main()");
  }
}
